@@ -8,8 +8,8 @@ import asyncio
 import hmac
 import logging
 import re
-from typing import (Dict, Any, Optional, AnyStr, Callable, Union, List,
-                    Awaitable, Coroutine)
+from typing import (Dict, Any, Optional, AnyStr, Callable, Union, Awaitable,
+                    Coroutine)
 
 try:
     import ujson as json
@@ -24,7 +24,8 @@ from .bus import EventBus
 from .exceptions import Error, TimingError
 from .event import Event
 from .message import Message, MessageSegment
-from .utils import ensure_async
+from .utils import ensure_async, run_async_funcs
+from .typing import Message_T
 
 from . import exceptions
 from .exceptions import *  # noqa: F401, F403
@@ -37,15 +38,17 @@ __all__ += exceptions.__all__
 __pdoc__ = {}
 
 
-def _deco_maker(type_: str) -> Callable:
+def _deco_maker(deco_method: Callable, type_: str) -> Callable:
     def deco_deco(self, arg: Optional[Union[str, Callable]] = None,
                   *sub_event_names: str) -> Callable:
         def deco(func: Callable) -> Callable:
             if isinstance(arg, str):
                 e = [type_ + '.' + e for e in [arg] + list(sub_event_names)]
-                self.on(*e)(func)
+                # self.on(*e)(func)
+                deco_method(self, *e)(func)
             else:
-                self.on(type_)(func)
+                # self.on(type_)(func)
+                deco_method(self, type_)(func)
             return func
 
         if isinstance(arg, Callable):
@@ -117,6 +120,7 @@ class CQHttp(AsyncApi):
         self._api = UnifiedApi()
         self._sync_api = None
         self._bus = EventBus()
+        self._before_sending_funcs = set()
         self._loop = None
 
         self._server_app = Quart(import_name, **(server_app_kwargs or {}))
@@ -218,8 +222,7 @@ class CQHttp(AsyncApi):
         """
         return await self._api.call_action(action=action, **params)
 
-    async def send(self, event: Event,
-                   message: Union[str, Dict[str, Any], List[Dict[str, Any]]],
+    async def send(self, event: Event, message: Message_T,
                    **kwargs) -> Optional[Dict[str, Any]]:
         """
         向触发事件的主体发送消息。
@@ -228,12 +231,14 @@ class CQHttp(AsyncApi):
         命名参数用于控制是否 at 事件的触发者，默认为 `False`。其它命名参数作为
         CQHTTP API ``send_msg`` 的参数直接传递。
         """
-        at_sender = kwargs.pop('at_sender', False) and 'user_id' in event
+        msg = message if isinstance(message, Message) else Message(message)
+        await run_async_funcs(self._before_sending_funcs, event, msg, kwargs)
 
-        keys = {'message_type', 'user_id', 'group_id', 'discuss_id',
-                'auto_escape'}
+        at_sender = kwargs.pop('at_sender', False) and ('user_id' in event)
+
+        keys = {'message_type', 'user_id', 'group_id', 'discuss_id'}
         params = {k: v for k, v in event.items() if k in keys}
-        params['message'] = message
+        params['message'] = msg
         params.update(kwargs)
 
         if 'message_type' not in params:
@@ -250,6 +255,23 @@ class CQHttp(AsyncApi):
 
         return await self.send_msg(**params)
 
+    def before_sending(self, func: Callable) -> Callable:
+        """
+        注册发送消息前的钩子函数，用作装饰器，例如：
+
+        ```py
+        @bot.before_sending
+        async def hook(event: Event, message: Message, kwargs: Dict[str, Any]):
+            message.clear()
+            message.append(MessageSegment.text('hooked!'))
+        ```
+
+        该钩子函数在刚进入 `CQHttp.send` 函数时运行，用户可在钩子函数中修改要发送的
+        ``message`` 和发送参数 ``kwargs``。
+        """
+        self._before_sending_funcs.add(ensure_async(func))
+        return func
+
     def subscribe(self, event_name: str, func: Callable) -> None:
         """注册事件处理函数。"""
         self._bus.subscribe(event_name, ensure_async(func))
@@ -299,11 +321,11 @@ class CQHttp(AsyncApi):
 
         return deco
 
-    on_message = _deco_maker('message')
+    on_message = _deco_maker(on, 'message')
     __pdoc__['CQHttp.on_message'] = """
     注册消息事件处理函数，用作装饰器，例如：
 
-    ```
+    ```py
     @bot.on_message('private')
     async def handler(event):
         pass
@@ -311,7 +333,7 @@ class CQHttp(AsyncApi):
 
     这等价于：
 
-    ```
+    ```py
     @bot.on('message.private')
     async def handler(event):
         pass
@@ -319,22 +341,117 @@ class CQHttp(AsyncApi):
 
     也可以不加参数，表示注册为所有消息事件的处理函数，例如：
 
-    ```
+    ```py
     @bot.on_message
     async def handler(event):
         pass
     ```
     """
 
-    on_notice = _deco_maker('notice')
+    on_notice = _deco_maker(on, 'notice')
     __pdoc__['CQHttp.on_notice'] = "注册通知事件处理函数，用作装饰器，用法同上。"
 
-    on_request = _deco_maker('request')
+    on_request = _deco_maker(on, 'request')
     __pdoc__['CQHttp.on_request'] = "注册请求事件处理函数，用作装饰器，用法同上。"
 
-    on_meta_event = _deco_maker('meta_event')
+    on_meta_event = _deco_maker(on, 'meta_event')
     __pdoc__['CQHttp.on_meta_event'] = "注册元事件处理函数，用作装饰器，用法同上。"
 
+    def hook_before(self, event_name: str, func: Callable) -> None:
+        """注册事件处理前的钩子函数。"""
+        self._bus.hook_before(event_name, ensure_async(func))
+
+    def unhook_before(self, event_name: str, func: Callable) -> None:
+        """取消注册事件处理前的钩子函数。"""
+        self._bus.unhook_before(event_name, func)
+
+    def before(self, *event_names: str) -> Callable:
+        """
+        注册事件处理前的钩子函数，用作装饰器，例如：
+
+        ```py
+        @bot.before('notice.group_decrease', 'notice.group_increase')
+        async def hook(event):
+            pass
+        ```
+
+        参数为要注册的事件名，格式是点号分割的各级事件类型，见 `Event.name`。
+
+        钩子函数的注册方法和事件处理函数几乎完全一致，只需将 ``on`` 改为 ``before``。
+
+        各级 before 钩子函数全部运行完成后，才会运行事件处理函数。
+        """
+
+        def deco(func: Callable) -> Callable:
+            for name in event_names:
+                self.hook_before(name, func)
+            return func
+
+        return deco
+
+    before_message = _deco_maker(before, 'message')
+    __pdoc__['CQHttp.before_message'] = """
+    注册消息事件处理前的钩子函数，用作装饰器，例如：
+
+    ```py
+    @bot.before_message('private')
+    async def hook(event):
+        pass
+    ```
+
+    这等价于：
+
+    ```py
+    @bot.before('message.private')
+    async def hook(event):
+        pass
+    ```
+
+    也可以不加参数，表示注册为所有消息事件处理前的钩子函数，例如：
+
+    ```py
+    @bot.before_message
+    async def hook(event):
+        pass
+    ```
+    """
+
+    before_notice = _deco_maker(before, 'notice')
+    __pdoc__['CQHttp.before_notice'] = "注册通知事件处理前的钩子函数，用作装饰器，用法同上。"
+
+    before_request = _deco_maker(before, 'request')
+    __pdoc__['CQHttp.before_request'] = "注册请求事件处理前的钩子函数，用作装饰器，用法同上。"
+
+    before_meta_event = _deco_maker(before, 'meta_event')
+    __pdoc__['CQHttp.before_meta_event'] = "注册元事件处理前的钩子函数，用作装饰器，用法同上。"
+
+    def on_startup(self, func: Callable) -> Callable:
+        """
+        注册 bot 启动时钩子函数，用作装饰器，例如：
+
+        ```py
+        @bot.on_startup
+        async def startup():
+            await db.init()
+        ```
+        """
+        return self.server_app.before_serving(func)
+
+    def on_websocket_connection(self, func: Callable) -> Callable:
+        """
+        注册 WebSocket 连接元事件处理函数，等价于 ``on_meta_event('lifecycle.connect')``，例如：
+
+        ```py
+        @bot.on_websocket_connection
+        async def handler(event):
+            global groups
+            groups = await bot.get_group_list(self_id=event.self_id)
+        ```
+
+        注：仅支持 CQHTTP v4.14+。
+        """
+        return self.on_meta_event('lifecycle.connect')(func)
+
     async def _handle_http_event(self) -> Response:
         if self._secret:
             if 'X-Signature' not in request.headers:
