@@ -3,7 +3,7 @@
 """
 
 import asyncio
-from typing import (Any, Callable, Awaitable)
+from typing import (Any, Callable, Awaitable, Iterable, List)
 
 from quart.utils import run_sync
 
@@ -26,3 +26,17 @@ def sync_wait(coro: Awaitable[Any],
     """
     fut = asyncio.run_coroutine_threadsafe(coro, loop)
     return fut.result()
+
+
+async def run_async_funcs(funcs: Iterable[Callable[..., Awaitable[Any]]],
+                          *args, **kwargs) -> List[Any]:
+    """
+    同时运行多个异步函数，并等待所有函数运行完成，返回运行结果列表。
+    """
+    results = []
+    coros = []
+    for f in funcs:
+        coros.append(f(*args, **kwargs))
+    if coros:
+        results += await asyncio.gather(*coros)
+    return results
